#!/usr/bin/python2
# -*- coding: utf8 -*-

#-----------------------------------------
# Name:         pylote.pyw
# Version:      1.2
# Author:       Pascal Peter
# Copyright:    (C) 2009-2014 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr
# Email:        pascal.peter at free.fr
#-----------------------------------------


################################################################################
# Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le
# modifier conformément aux dispositions de la Licence Publique Générale GNU,
# telle que publiée par la Free Software Foundation ; version 3 de la licence,
# ou encore toute version ultérieure.
# 
# Ce programme est distribué dans l'espoir qu'il sera utile,
# mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION
# ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail,
# voir la Licence Publique Générale GNU.
# 
# Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même
# temps que ce programme ; si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.
# -------------------------------------------------------------------------------
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
################################################################################


"""
DESCRIPTION :
    Fichier de lancement du logiciel.
    Contient l'interface graphique.
"""


# importation des modules utiles :
from __future__ import division, print_function
import sys
import os

# récupération du chemin :
try:
    HERE = os.path.dirname(os.path.abspath(__file__))
except:
    HERE = os.path.dirname(sys.argv[0])
# ajout du chemin au path (+ libs) :
sys.path.insert(0, HERE)
sys.path.insert(0, HERE + os.sep + 'libs')
# on démarre dans le bon dossier :
os.chdir(HERE)

# importation des modules perso :
import utils, utils_config, utils_dialogs, utils_instruments

# PyQt4 ou PySide :
if utils.PYSIDE:
    from PySide import QtCore, QtGui, QtSvg
else:
    from PyQt4 import QtCore, QtGui, QtSvg












"""
###########################################################
###########################################################

                LE GRAPHICSVIEW : ZONE D'AFFICHAGE

###########################################################
###########################################################
"""

class GraphicsView(QtGui.QGraphicsView):
    """
    Le GraphicsView qui affiche l'image de fond et les items.
    """
    def __init__(self, parent):
        QtGui.QGraphicsView.__init__(self, parent)
        
        """
        on récupère main (fenêtre principale)
        pas d'ascenseurs
        """
        self.main = parent

        self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)

        self.isDrawingEnabled = False
        self.scribbling = False
        self.attachToInstrument = None
        self.lastPoint = QtCore.QPointF()
        self.pens = []
        for i in range(5):
            newPen = QtGui.QPen(QtCore.Qt.black, 1,
                                  QtCore.Qt.SolidLine, QtCore.Qt.RoundCap,
                                  QtCore.Qt.RoundJoin)
            self.pens.append(newPen)
        self.drawPen = self.pens[0]
        self.path = QtGui.QPainterPath()
        self.font = QtGui.QFont()
        self.actualCursor = QtCore.Qt.ArrowCursor
        self.drawId = -1

    def keyPressEvent(self, event):
        """
        Gestion des touches Control et Delete
        """
        if event.key() == QtCore.Qt.Key_Control:
            self.main.toolsWindow.changeTrace(newMode=True)
        elif event.key() == QtCore.Qt.Key_Delete:
            self.main.removeSelected()
        else:
            self.main.keyPressEvent(event)
            
    def keyReleaseEvent(self, event):
        """
        Gestion de la touche Control
        """
        if event.key() == QtCore.Qt.Key_Control:
            self.main.toolsWindow.changeTrace(newMode=False)

    def mousePressEvent(self, event):
        """
        On veut distinguer le clic du double-clic.
        Ici, on met la variable doubleClick à False,
            et on retarde la gestion du simple clic à la fonction doMouseClick.
        Entre temps, si c'est un double-clic, la variable doubleClick sera
            devenue True.
        """
        QtGui.QGraphicsView.mousePressEvent(self, event)
        self.doubleClick = False
        self.mouseEventButton, self.mouseEventPos = event.button(), self.mapToScene(event.pos())
        self.isDrawingEnabled = False
        # en dessous de 300, ça foire :
        QtCore.QTimer.singleShot(300, self.doMouseClick)

    def mouseDoubleClickEvent(self, event):
        """
        On met la variable doubleClick à True, pour ne pas executer 
            la procédure liée au simple clic.
        Si le double-clic est fait sur un texte ou le label d'un point,
            il permet de l'éditer.
        Sinon, il permet de cacher/afficher la barre d'outils.
        """
        QtGui.QGraphicsView.mouseDoubleClickEvent(self, event)
        self.doubleClick = True
        if self.main.editText() == False:
            if not(self.main.toolsWindow.toolsKidMode):
                if self.main.toolsWindow.isVisible():
                    self.main.doMinimizeToolsWindow()
                else:
                    self.main.doRestoreToolsWindow()

    def doMouseClick(self):
        """
         Si la variable doubleClick est à False, on fait bien un simple clic.
         Gestion des différents cas possibles 
            (ajout d'un texte, début d'un tracé, ...)
        """
        if self.doubleClick:
            return
        #pour ne calculer qu'une seule fois (et pas à nouveau dans mouseMoveEvent) :
        self.isDrawingEnabled = self.main.isDrawingEnabled()
        if self.isDrawingEnabled:
            if self.mouseEventButton == QtCore.Qt.LeftButton and self.main.drawingMode[0] != 'NO':
                point = self.mouseEventPos
                self.attachToInstrument = None
                if self.main.drawingMode[0] in ('POINT', 'LINE'):
                    for instrument in self.main.listeInstruments[:3]:
                        if (self.attachToInstrument == None) and instrument.isVisible():
                            A = instrument.mapToScene(instrument.A)
                            B = instrument.mapToScene(instrument.B)
                            d = utils.distanceSegment(point, A, B)
                            if d < self.main.attachDistance:
                                self.attachToInstrument = instrument
                                point = utils.projectionOrthogonale(point, A, B)
                if self.main.drawingMode[0] == 'TEXT':
                    try:
                        self.main.toolsWindow.hide()
                        textItem = QtGui.QGraphicsTextItem()
                        textItem.setFont(self.font)
                        textItem.setDefaultTextColor(self.drawPen.color())
                        dialog = utils_dialogs.TextItemDlg(parent=self, graphicsTextItem=textItem)
                        if dialog.exec_() == QtGui.QDialog.Accepted:
                            self.scene().addItem(textItem)
                            textItem.setPos(point)
                            self.drawId += 1
                            textItem.setData(QtCore.Qt.UserRole, self.drawId)
                            self.main.saveTempFile()
                    finally:
                        self.main.toolsWindow.show()
                elif self.main.drawingMode[0] == 'POINT':
                    try:
                        self.main.toolsWindow.hide()
                        brush = QtGui.QBrush(self.drawPen.color())
                        exStyle = self.drawPen.style()
                        self.drawPen.setStyle(QtCore.Qt.SolidLine)
                        pointItem = utils_instruments.PointItem(self.main, self.drawPen, brush, self.font)
                        self.drawPen.setStyle(exStyle)
                        ok = pointItem.chooseText(self.drawPen, brush, self.font)
                        if ok:
                            self.scene().addItem(pointItem)
                            pointItem.setPos(point)
                            self.drawId += 1
                            pointItem.setData(QtCore.Qt.UserRole, self.drawId)
                            self.main.saveTempFile()
                    finally:
                        self.main.toolsWindow.show()
                elif self.main.drawingMode[0] == 'PIXMAP':
                    try:
                        self.main.toolsWindow.hide()
                        fileName = QtGui.QFileDialog.getOpenFileName(
                            self.main, 
                            QtCore.QCoreApplication.translate('MainWindow', 'Open Image'),
                            self.main.pixmapDir, 
                            QtCore.QCoreApplication.translate(
                                'MainWindow', 'Image Files (*.png *.jpg *.jpeg *.xpm)'))
                        # pour PySide :
                        if isinstance(fileName, tuple):
                            fileName = fileName[0]
                        if fileName != '':
                            self.main.pixmapDir = QtCore.QFileInfo(fileName).absolutePath()
                            image = QtGui.QImage(fileName)
                            pixmap = QtGui.QPixmap.fromImage(image)
                            pixmapItem = QtGui.QGraphicsPixmapItem(pixmap)
                            self.scene().addItem(pixmapItem)
                            pixmapItem.setPos(point)
                            self.drawId += 1
                            pixmapItem.setData(QtCore.Qt.UserRole, self.drawId)
                            self.main.saveTempFile()
                    finally:
                        self.main.toolsWindow.show()
                elif self.main.drawingMode[0] == 'PASTE':
                    try:
                        self.main.toolsWindow.hide()
                        # récupération du presse-papier :
                        clipboard = QtGui.QApplication.clipboard()
                        mimeData = clipboard.mimeData()
                        if mimeData.hasImage():
                            pixmap = clipboard.pixmap()
                            pixmapItem = QtGui.QGraphicsPixmapItem(pixmap)
                            self.scene().addItem(pixmapItem)
                            pixmapItem.setPos(point)
                            self.drawId += 1
                            pixmapItem.setData(QtCore.Qt.UserRole, self.drawId)
                            self.main.saveTempFile()
                        elif mimeData.hasText():
                            textItem = QtGui.QGraphicsTextItem()
                            textItem.setFont(self.font)
                            textItem.setDefaultTextColor(self.drawPen.color())
                            textItem.setPlainText(clipboard.text())
                            self.scene().addItem(textItem)
                            textItem.setPos(point)
                            self.drawId += 1
                            textItem.setData(QtCore.Qt.UserRole, self.drawId)
                            self.main.saveTempFile()
                    finally:
                        self.main.toolsWindow.show()
                elif self.main.drawingMode[0] == 'LINE':
                    self.scribbling = True
                    self.last = None
                    self.initLine(point)
                else: 
                    self.scribbling = True
                    self.last = None
                    self.initCurve(point)

    def mouseMoveEvent(self, event):
        """
        D'une part, si on tace une courbe, on met à jour.
        D'autre part, on vérifie quel curseur est affiché (y compris le faux)
        """
        QtGui.QGraphicsView.mouseMoveEvent(self, event)
        if self.isDrawingEnabled:
            point = self.mapToScene(event.pos())
            if (event.buttons() and QtCore.Qt.LeftButton and self.scribbling):
                if self.main.drawingMode[0] == 'LINE':
                    if self.attachToInstrument != None:
                        A = self.attachToInstrument.mapToScene(self.attachToInstrument.A)
                        B = self.attachToInstrument.mapToScene(self.attachToInstrument.B)
                        H = utils.projectionOrthogonale(point, A, B)
                        self.drawLineTo(H)
                    else:
                        self.drawLineTo(point)
                else:
                    self.drawCurveTo(point)
        if utils_instruments.WITH_FALSE_CURSOR:
            point = self.mapToScene(event.pos())
            self.main.myCursor.setPos(point.x(), point.y())
            if self.cursor() != QtCore.Qt.BlankCursor:
                self.setCursor(QtCore.Qt.BlankCursor)
        elif self.cursor() != self.actualCursor:
            self.setCursor(self.actualCursor)

    def mouseReleaseEvent(self, event):
        """
        On termine l'éventuel tracé
        """
        QtGui.QGraphicsView.mouseReleaseEvent(self, event)
        if self.isDrawingEnabled:
            if (event.button() == QtCore.Qt.LeftButton) and self.scribbling:
                point = self.mapToScene(event.pos())

                if self.main.drawingMode[0] == 'LINE':
                    if self.attachToInstrument != None:
                        A = self.attachToInstrument.mapToScene(self.attachToInstrument.A)
                        B = self.attachToInstrument.mapToScene(self.attachToInstrument.B)
                        H = utils.projectionOrthogonale(point, A, B)
                        item = self.drawLineTo(H)
                    else:
                        item = self.drawLineTo(point)
                else:
                    item = self.drawCurveTo(point, endLine=True)
                self.drawId += 1
                item.setData(QtCore.Qt.UserRole, self.drawId)
                self.scribbling = False
                self.main.saveTempFile()
        elif (self.main.drawingMode[0] == 'SELECT') and (len(self.scene().selectedItems())):
            self.main.saveTempFile()

    def initLine(self, startPoint):
        """
        Début d'une ligne (un segment).
        On initialise le path et le pathItem
        """
        self.lastPoint = startPoint
        self.path = QtGui.QPainterPath()
        self.path.moveTo(startPoint)
        pen = self.drawPen
        brush = QtGui.QBrush(QtCore.Qt.green, QtCore.Qt.NoBrush)
        self.pathItem = self.scene().addPath(self.path, pen, brush)
        self.pathItem.setFlags(QtGui.QGraphicsItem.ItemIgnoresTransformations)

    def drawLineTo(self, endPoint):
        """
        On supprime systématiquement la ligne précédente
            avant d'ajouter la nouvelle
        """
        self.path.lineTo(endPoint)
        if self.last != None:
            self.scene().removeItem(self.last)
        self.last = self.scene().addLine(self.lastPoint.x(), self.lastPoint.y(), 
                                            endPoint.x(), endPoint.y(), 
                                            self.drawPen)
        return self.last

    def initCurve(self, startPoint):
        """
        Début d'une courbe.
        On initialise le path et le pathItem. En cours de tracé, on affichera 
            une suite de segments (polyligne).
        Le path sera recalculé à la fin du tracé, pour tracer une courbe.
        On récupèrera donc les points de passage dans une liste (listPoints).
        """
        self.lastPoint = startPoint
        self.path = QtGui.QPainterPath()
        self.path.moveTo(startPoint)
        if self.main.drawingMode[0] == 'HIGHLIGHTER':
            pen = QtGui.QPen(self.main.drawingMode[1], 20,
                QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
        else:
            pen = self.drawPen
        brush = QtGui.QBrush(QtCore.Qt.green, QtCore.Qt.NoBrush)
        self.pathItem = self.scene().addPath(self.path, pen, brush)
        self.pathItem.setFlags(QtGui.QGraphicsItem.ItemIgnoresTransformations)
        # pour faire la courbe finale :
        self.listPoints = []
        self.listPoints.append(startPoint)

    def drawCurveTo(self, endPoint, endLine=False):
        """
        En cours de tracé, c'est path qui est affiché.
            On récupère le point dans listPoints.
        À la fin (appel avec la variable endLine à True), 
            on calcule la courbe, et on l'affiche.
        """
        self.path.lineTo(endPoint)
        self.lastPoint = endPoint
        self.pathItem.setPath(self.path)
        self.listPoints.append(endPoint)
        if endLine:
            """
            # tentative d'interpolation (désactivé car moche)
            self.lastPoint = endPoint
            self.path = QtGui.QPainterPath()
            self.path.moveTo(self.listPoints[0])
            for i in range(len(self.listPoints) // 2):
                self.path.quadTo(self.listPoints[2*i], self.listPoints[2*i + 1])
            self.pathItem.setPath(self.path)
            """
            return self.pathItem

    def initArc(self, center, startPoint):
        """
        Début d'un arc de compas.
        On initialise les différentes valeurs d'angles utiles.
        On calcule le rectangle d'affichage (carré contenant le cercle complet).
        """
        self.lastPoint = startPoint
        self.path = QtGui.QPainterPath()
        self.path.moveTo(startPoint)
        pen = self.drawPen
        brush = QtGui.QBrush(QtCore.Qt.green, QtCore.Qt.NoBrush)
        self.pathItem = self.scene().addPath(self.path, pen, brush)
        self.startPoint = startPoint
        rayon = QtCore.QLineF(center, startPoint)
        self.startAngle = rayon.angle()
        self.minAngle = self.startAngle
        self.maxAngle = self.startAngle
        self.lastAngle = self.startAngle
        length = rayon.length()
        self.rectangle = QtCore.QRectF(center.x() - length, center.y() - length, 
                                    2 * length, 2 * length)

    def drawArcTo(self, center, endPoint):
        """
        En cours de tracé, on recalcule les différents angles utiles.
            Pas mal de cas à gérer, et la complication du passage brutal de 0 à 360.
        Ensuite on trace l'arc.
        """
        self.lastPoint = QtCore.QPointF(endPoint)
        rayon = QtCore.QLineF(center, endPoint)
        self.path = QtGui.QPainterPath()
        angle = rayon.angle()
        if angle - self.lastAngle > 300:
            self.minAngle = self.minAngle + 360.0
            self.maxAngle = self.maxAngle + 360.0
        if angle - self.lastAngle < -300:
            self.minAngle = self.minAngle - 360.0
            self.maxAngle = self.maxAngle - 360.0
        if angle < self.minAngle:
            self.minAngle = angle
            self.startPoint = self.lastPoint
        if angle > self.maxAngle:
            self.maxAngle = angle
        endAngle = self.maxAngle - self.minAngle
        self.path.moveTo(self.startPoint)
        self.path.arcTo(self.rectangle, self.minAngle, endAngle)
        self.pathItem.setPath(self.path)
        self.lastAngle = angle
        return self.pathItem

    def penColor(self):
        """
        Dialog de sélection de couleur
        """
        try:
            self.main.toolsWindow.hide()
            color = self.drawPen.color()
            newColor = QtGui.QColorDialog.getColor(
                color, 
                self, 
                QtCore.QCoreApplication.translate('GraphicsView', 'Select color'), 
                QtGui.QColorDialog.ShowAlphaChannel)
            if newColor.isValid():
                self.drawPen.setColor(newColor)
                #print(newColor, newColor.red(), newColor.green(), newColor.blue(), newColor.alpha())
        finally:
            self.main.toolsWindow.show()

    def penWidth(self):
        """
        Dialog de sélection d'épaisseur du crayon
        """
        try:
            self.main.toolsWindow.hide()
            newWidth, ok = QtGui.QInputDialog.getInteger(
                self, utils.PROGLABEL,
                QtCore.QCoreApplication.translate('GraphicsView', 'Select pen width:'),
                self.drawPen.width(),
                1, 50, 1)
            if ok:
                self.drawPen.setWidth(newWidth)
        finally:
            self.main.toolsWindow.show()

    def penStyle(self):
        """
        Dialog de sélection de style du crayon
        """
        try:
            self.main.toolsWindow.hide()
            solid = QtCore.QCoreApplication.translate('GraphicsView', 'Solid')
            dash = QtCore.QCoreApplication.translate('GraphicsView', 'Dash')
            dot = QtCore.QCoreApplication.translate('GraphicsView', 'Dot')
            dashDot = QtCore.QCoreApplication.translate('GraphicsView', 'Dash Dot')
            dashDotDot = QtCore.QCoreApplication.translate('GraphicsView', 'Dash Dot Dot')
            items = {solid: QtCore.Qt.SolidLine, 
                    dash: QtCore.Qt.DashLine, 
                    dot: QtCore.Qt.DotLine,
                    dashDot: QtCore.Qt.DashDotLine, 
                    dashDotDot: QtCore.Qt.DashDotDotLine}
            reponse, ok = QtGui.QInputDialog.getItem(
                self, utils.PROGLABEL,
                QtCore.QCoreApplication.translate('GraphicsView', 'Select pen style:'), 
                [solid, dash, dot, dashDot, dashDotDot], 
                0, False)
            if ok and reponse != '':
                newStyle = items[reponse]
                self.drawPen.setStyle(newStyle)
        finally:
            self.main.toolsWindow.show()

    def comboBoxPensChanged(self):
        """
        On change de stylo
        """
        indexPen = self.main.toolsWindow.comboBoxPens.currentIndex()
        self.drawPen = self.pens[indexPen]

    def customColor(self):
        if self.main.toolsWindow.actionEditCustomColors.isChecked():
            try:
                self.main.toolsWindow.hide()
                color = self.sender().data()
                newColor = QtGui.QColorDialog.getColor(
                    color, 
                    self, 
                    QtCore.QCoreApplication.translate('GraphicsView', 'Select color'), 
                    QtGui.QColorDialog.ShowAlphaChannel)
                if newColor.isValid():
                    self.drawPen.setColor(newColor)
                    pixmap = QtGui.QPixmap(128, 128)
                    pixmap.fill(newColor)
                    icon = QtGui.QIcon(pixmap)
                    self.sender().setIcon(icon)
                    self.sender().setData(newColor)
            finally:
                self.main.toolsWindow.show()
        else:
            color = self.sender().data()
            self.drawPen.setColor(color)

    def customSize(self):
        if self.main.toolsWindow.actionEditCustomSizes.isChecked():
            try:
                self.main.toolsWindow.hide()
                width = self.sender().data()
                newWidth, ok = QtGui.QInputDialog.getInteger(
                    self, utils.PROGLABEL,
                    QtCore.QCoreApplication.translate('GraphicsView', 'Select pen width:'),
                    width,
                    1, 50, 1)
                if ok:
                    self.drawPen.setWidth(newWidth)
                    self.sender().setData(newWidth)
                    self.sender().setToolTip('{0}'.format(newWidth))
                    # on réorganise pour garder l'ordre croissant :
                    sizes = []
                    for action in self.main.toolsWindow.actionCustomSizes:
                        sizes.append(action.data())
                    sizes = sorted(sizes)
                    for i in range(5):
                        action = self.main.toolsWindow.actionCustomSizes[i]
                        size = sizes[i]
                        action.setData(size)
                        action.setToolTip('{0}'.format(size))
            finally:
                self.main.toolsWindow.show()
        else:
            penWidth = self.sender().data()
            self.drawPen.setWidth(penWidth)











"""
###########################################################
###########################################################

                LA FENÊTRE PRINCIPALE

###########################################################
###########################################################
"""

class MainWindow(QtGui.QMainWindow):
    def __init__(self, translator, parent=None):
        QtGui.QMainWindow.__init__(self, parent)
        """
        mise en place de l'interface
        plus des variables utiles
        """

        # """
        dd = QtGui.QApplication.desktop()
        print('screenCount : ', dd.screenCount())
        print('primaryScreen : ', dd.primaryScreen())
        print('isVirtualDesktop : ', dd.isVirtualDesktop())
        print('availableGeometry : ', dd.availableGeometry())
        for i in range(dd.screenCount()):
            print(i, 'availableGeometry : ', dd.availableGeometry(i))
        print('screenGeometry : ', dd.screenGeometry())
        for i in range(dd.screenCount()):
            print(i, 'screenGeometry : ', dd.screenGeometry(i))
        print('winId : ', dd.winId())
        # """

        # Les chemins et autres
        self.beginDir = QtCore.QDir.currentPath()
        self.filesDir = QtCore.QDir.homePath()
        self.appExt = QtCore.QCoreApplication.translate('MainWindow', 'Pylote Files (*.plt)')
        self.backgroundsDir = self.beginDir + '/images/backgrounds'
        self.pixmapDir = self.beginDir + '/images'
        self.tempPath = utils.createTempAppDir()
        self.tempFileNum = -1

        # Un premier Screenshot avant affichage de la fenêtre
        self.backgroundPixmap = QtGui.QPixmap.grabWindow(QtGui.QApplication.desktop().winId())

        # Pour la fenêtre principale, l'i18n et on lit les Settings
        self.defaultFlags = self.windowFlags()
        self.setWindowFlags(self.defaultFlags | QtCore.Qt.FramelessWindowHint)
        self.setWindowTitle(utils.PROGLABEL)
        self.translator = translator
        self.lang = LANG

        # Pour l'affichage de l'image (QGraphicsScene + GraphicsView)
        scene = QtGui.QGraphicsScene(self)
        self.view = GraphicsView(self)
        scene.setSceneRect(0, 0, 1, 1)
        self.view.setScene(scene)
        self.setCentralWidget(self.view)

        # Les instruments, dans une liste (y compris le faux curseur)
        self.ruler = utils_instruments.Ruler(self)
        self.square = utils_instruments.Square(self)
        self.squareNotGraduated = utils_instruments.SquareNotGraduated(self)
        self.protractor = utils_instruments.Protractor(self)
        self.compass = utils_instruments.Compass(self)
        self.myCursor = utils_instruments.MyCursor(self)
        self.listeInstruments = (self.ruler, self.square, self.squareNotGraduated,
            self.protractor, self.compass, self.myCursor)
        for instrument in self.listeInstruments:
            self.view.scene().addItem(instrument)
            instrument.setVisible(False)
        self.attachDistance = 20

        self.readSettings()

        # La boîte à outils :
        self.toolsWindow = utils_dialogs.ToolsWindow(self)
        self.toolsWindow.show()
        self.toolsWindow.restoreGeometry(self.toolsWindowGeometry)
        self.toolsWindow.restoreState(self.toolsWindowState)
        self.toolsWindow.move(120, 80)
        self.firstRestore = [True, True]

        # Les actions et la TrayIcon
        self.createActions()
        if utils.WITH_TRAY_ICON:
            self.createTrayIcon()
            self.trayIcon.setIcon(QtGui.QIcon('./images/icon.png'))
            self.trayIcon.show()
            QtCore.QObject.connect(self.trayIcon,
                    QtCore.SIGNAL('activated(QSystemTrayIcon::ActivationReason)'),
                    self.iconActivated)

        # divers
        self.brush = QtGui.QBrush()
        self.drawingMode = ('NO', None)
        self.IsMinimized = False

        self.fileName = ''
        # recherche d'un éventuel fichier plt passé en argument :
        for arg in sys.argv:
            extension = arg.split('.')[-1]
            if extension == 'plt':
                self.fileName = utils.u(arg)

        QtCore.QTimer.singleShot(1, self.firstScreenShot)
        self.toolsWindow.setFocus()

    def resizeEvent(self, event):
        """
        gestion du décalage de l'image à afficher
        et affichage d'icelle
        """
        self.calculDecalage()
        brushTransform = QtGui.QTransform()
        brushTransform.translate(-self.decalX, -self.decalY)
        self.brush.setTransform(brushTransform)
        self.brush.setTexture(self.backgroundPixmap)
        self.view.scene().setBackgroundBrush(self.brush)
        # la suite a l'air nécessaire sous windows pour enlever les bordures :
        if sys.platform == 'win32':
            maskedRegion = QtGui.QRegion(0, 0, self.width(), self.height(), 
                                        QtGui.QRegion.Rectangle)
            self.setMask(maskedRegion)

    def calculDecalage(self):
        """
        Calcul du décalage pour positionner la fenêtre
            et l'image à afficher
        Encore quelques bugs sur certains postes (windows !!!)
        """
        if utils.SCREEN_MODE == 'TESTS':
            rect = QtGui.QApplication.desktop().availableGeometry(self.screenNumber)
            self.decalX = rect.x() + (rect.width() - self.width()) // 2
            self.decalY = rect.y() + (rect.height() - self.height()) // 2
            self.move(self.decalX, self.decalY)
            self.decalX = self.decalX + self.width() // 2 - self.view.getContentsMargins()[0] // 2
            self.decalY = self.decalY + self.height() // 2 - self.view.getContentsMargins()[1] // 2
        elif utils.SCREEN_MODE == 'FULL_SPACE':
            # Position de la fenêtre en fonction de l'OS
            rect = QtGui.QApplication.desktop().availableGeometry(self.screenNumber)
            if sys.platform == 'win32':
                self.decalX = rect.x()
                self.decalY = rect.y()
                # cas windows géré à part :
                self. moveIfWin()
            else:
                self.decalX = rect.x() + (rect.width() - self.width()) // 2
                self.decalY = rect.y() + (rect.height() - self.height()) // 2                
                self.move(self.decalX, self.decalY)
            self.decalX = self.decalX + self.width() // 2 - self.view.getContentsMargins()[0] // 2
            self.decalY = self.decalY + self.height() // 2 - self.view.getContentsMargins()[1] // 2
        elif utils.SCREEN_MODE == 'FULL_SCREEN':
            self.decalX = self.width() // 2
            self.decalY = self.height() // 2
        #print('calculDecalage', self.decalX, self.decalY)

    def moveIfWin(self):
        """
        En mode FULL_SPACE, je n'arrive pas à placer correctement la fenêtre sous windows.
        Pour ce système, il vaut mieux utiliser le mode FULL_SCREEN.
        Cette procédure est donc à améliorer.
        Sous Linux, ça marche du tonnerre...
        """
        if utils.SCREEN_MODE == 'FULL_SPACE':
            origine = self.mapToGlobal(QtCore.QPoint(0, 0))
            #print(origine)
            rect = QtGui.QApplication.desktop().availableGeometry(self.screenNumber)
            """
            #print(self.view.getContentsMargins()[0])
            marge = (3 * self.view.getContentsMargins()[0],
                     3 * self.view.getContentsMargins()[1])
            #print(marge)
            self.move(rect.x() - origine.x() - marge[0],
                      rect.y() - origine.y() - marge[1])
            """
            # pourquoi 4 ?  Ça ne marche forcément pas tout le temps.
            a = 4
            self.move(rect.x() - origine.x() - a,
                      rect.y() - origine.y() - a)

    def doMinimize(self):
        """
        Minimise tout le programme.
        """
        self.doMinimizeToolsWindow()
        self.hide()
        self.IsMinimized = True

    def doMinimizeToolsWindow(self):
        """
        Minimise la ToolsWindow.
        Séparé de la procédure précédente car peut être appelé 
            indépendamment (double-clic par exemple).
        """
        self.toolsWindow.hide()

    def doRestore(self):
        """
        Restaure tout le programme.
        Gestion du cas windows.
        """
        self.show()
        self.doRestoreToolsWindow()
        self.IsMinimized = False
        if sys.platform == 'win32':
            self.moveIfWin()

    def doRestoreToolsWindow(self):
        #print('doRestoreToolsWindow', self.toolsWindow.toolsKidMode, self.firstRestore)
        if self.toolsWindow.toolsKidMode:
            if self.firstRestore[1]:
                self.firstRestore[1] = False
        else:
            if self.firstRestore[0]:
                self.firstRestore[0] = False
                geometry = self.toolsWindow.geometry()
                frameGeometry = self.toolsWindow.frameGeometry()
                deltaX = frameGeometry.x() - geometry.x()
                deltaY = frameGeometry.y() - geometry.y()
                self.toolsWindow.move(geometry.x() + deltaX, geometry.y() + deltaY)
        self.toolsWindow.show()

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_K:
            self.switchKidMode()

    def switchKidMode(self):
        self.toolsWindow.toolsKidMode = not self.toolsWindow.toolsKidMode
        self.toolsWindow.hide()
        if self.toolsWindow.toolsKidMode:
            self.toolsWindowGeometry = self.toolsWindow.saveGeometry()
            self.toolsWindowState = self.toolsWindow.saveState()
            self.toolsWindow.setWindowFlags(self.toolsWindow.kidFlags)
            app.processEvents()
            self.toolsWindow.reloadToolBars()
            self.toolsWindow.restoreGeometry(self.toolsKidWindowGeometry)
            self.toolsWindow.restoreState(self.toolsKidWindowState)
            self.toolsWindow.setMaximumWidth(self.toolsKidWindowWidth)
            self.toolsWindow.setMaximumHeight(self.toolsKidWindowHeight)
            self.toolsWindow.move(self.toolsKidWindowX, self.toolsKidWindowY)
        else:
            self.toolsKidWindowWidth = self.toolsWindow.width()
            self.toolsKidWindowHeight = self.toolsWindow.height()
            self.toolsKidWindowX = self.toolsWindow.x()
            self.toolsKidWindowY = self.toolsWindow.y()
            self.toolsKidWindowGeometry = self.toolsWindow.saveGeometry()
            self.toolsKidWindowState = self.toolsWindow.saveState()
            self.toolsWindow.setWindowFlags(self.toolsWindow.defaultFlags)
            app.processEvents()
            self.toolsWindow.reloadToolBars()
            self.toolsWindow.restoreGeometry(self.toolsWindowGeometry)
            self.toolsWindow.restoreState(self.toolsWindowState)
        self.doRestoreToolsWindow()
        self.toolsWindow.repaint()
        self.toolsWindow.setMaximumWidth(16777215)
        self.toolsWindow.setMaximumHeight(16777215)

    def newScreenshot(self):
        """
        On cache le programme, puis on lance le ScreenShot
        """
        self.doMinimize()
        QtCore.QTimer.singleShot(self.screenShotDelay, self.shootScreen)

    def shootScreen(self):
        """
        On fait le ScreenShot
        Puis on place l'image et on réaffiche
        """
        self.backgroundPixmap = QtGui.QPixmap.grabWindow(QtGui.QApplication.desktop().winId())
        brushTransform = QtGui.QTransform()
        brushTransform.translate(-self.decalX, -self.decalY)
        self.brush.setTransform(brushTransform)
        self.brush.setTexture(self.backgroundPixmap)
        self.view.scene().setBackgroundBrush(self.brush)
        self.doRestore()
        self.saveTempFile()

    def firstScreenShot(self):
        """
        On place l'image du premier ScreenShot
        """
        self.calculDecalage()
        brushTransform = QtGui.QTransform()
        brushTransform.translate(-self.decalX, -self.decalY)
        self.brush.setTransform(brushTransform)
        self.brush.setTexture(self.backgroundPixmap)
        self.view.scene().setBackgroundBrush(self.brush)
        if self.fileName != '':
            self.fileReload()
        self.saveTempFile()

    def quit(self):
        utils.emptyDir(QtCore.QDir.tempPath() + "/" + utils.PROGNAME)
        self.close()

    def closeEvent(self, event):
        """
        On sauvegarde les Settings en quittant.
        """
        self.writeSettings()

    def readSettings(self):
        """
        Récupération des réglages.
        On ouvre aussi le fichier interne (defaultSettings) en cas de nouvelle variable
        lors d'une mise à jour du logiciel.
        """
        defaultSettings = QtCore.QSettings('libs/config.conf', QtCore.QSettings.IniFormat)
        settings = QtCore.QSettings(utils.PROGNAME, 'config')
        version = float(settings.value('PROGVERSION', 0.0))
        if version < utils.PROGVERSION:
            settings = defaultSettings
        # variables générales :
        if utils.SCREEN_MODE == 'TESTS':
            size = settings.value('size', QtCore.QSize(600, 400))
            self.resize(size)
        self.screenNumber = int(settings.value('SCREEN_NUMBER', 0))
        utils.changeScreenNumber(self.screenNumber)
        if not(self.screenNumber < QtGui.QApplication.desktop().screenCount()):
            self.screenNumber = 0
        self.screenShotDelay = int(settings.value('ScreenShotDelay', 
            defaultSettings.value('ScreenShotDelay')))
        filesDir = settings.value('filesDir', '')
        if filesDir != '' and QtCore.QDir(filesDir).exists():
            self.filesDir = filesDir
        self.printMode = settings.value('PrintMode', 'FullPage')
        self.printOrientation = settings.value('PrintOrientation', 'Landscape')
        self.attachDistance = int(settings.value('attachDistance', 20))
        # position et état des barres d'outils :
        self.iconSize = settings.value('iconSize', defaultSettings.value('iconSize'))
        self.toolsWindowGeometry = settings.value('toolsWindowGeometry', 
            defaultSettings.value('toolsWindowGeometry'))
        self.toolsWindowState = settings.value('toolsWindowState', 
            defaultSettings.value('toolsWindowState'))
        # ToolsKidWindow :
        self.iconSizeKid = settings.value('iconSizeKid', defaultSettings.value('iconSizeKid'))
        self.toolsKidWindowGeometry = settings.value('toolsKidWindowGeometry',
            defaultSettings.value('toolsKidWindowGeometry'))
        self.toolsKidWindowState = settings.value('toolsKidWindowState', 
            defaultSettings.value('toolsKidWindowState'))
        self.toolsKidWindowWidth = int(settings.value('toolsKidWindowWidth', 250))
        self.toolsKidWindowHeight = int(settings.value('toolsKidWindowHeight', 100))
        self.toolsKidWindowX = int(settings.value('toolsKidWindowX', 100))
        self.toolsKidWindowY = int(settings.value('toolsKidWindowY', 100))
        # blocage des unités (règle et équerre) :
        unitsLocked = int(settings.value('unitsLocked', 0))
        self.unitsLocked = (unitsLocked == 1)
        self.instrumentsScale = float(settings.value('instrumentsScale', 1))
        # les pinceaux personnalisés :
        for i in range(5):
            a = 'pen{0}/'.format(i + 1)
            penWidth = int(settings.value(a + 'Width', 1))
            self.view.pens[i].setWidth(penWidth)
            try:
                penColor = QtGui.QColor(settings.value(a + 'Color'))
                self.view.pens[i].setColor(penColor)
            except:
                self.view.pens[i].setColor(QtGui.QColor())
            penStyle = int(settings.value(a + 'Style', 1))
            if penStyle == 1:
                penStyle = QtCore.Qt.SolidLine
            elif penStyle == 2:
                penStyle = QtCore.Qt.DashLine
            elif penStyle == 3:
                penStyle = QtCore.Qt.DotLine
            elif penStyle == 4:
                penStyle = QtCore.Qt.DashDotLine
            elif penStyle == 5:
                penStyle = QtCore.Qt.DashDotDotLine
            else:
                penStyle = QtCore.Qt.SolidLine
            self.view.pens[i].setStyle(penStyle)
        self.view.drawPen = self.view.pens[0]
        # police :
        font = QtGui.QFont(settings.value('font'))
        fontPointSize = int(settings.value('fontPointSize', 14))
        font.setPointSize(fontPointSize)
        self.view.font = font

    def writeSettings(self):
        """
        On sauvegarde les variables SCREEN_MODE, SCREEN_NUMBER etc...
        """
        settings = QtCore.QSettings(utils.PROGNAME, 'config')
        settings.setValue('PROGVERSION', utils.PROGVERSION)
        # variables générales :
        if utils.SCREEN_MODE != 'TESTS':
            settings.setValue('SCREEN_MODE', utils.SCREEN_MODE)
        settings.setValue('SCREEN_NUMBER', utils.SCREEN_NUMBER)
        settings.setValue('iconSize', self.iconSize)
        settings.setValue('ScreenShotDelay', self.screenShotDelay)
        settings.setValue('filesDir', self.filesDir)
        settings.setValue('PrintMode', self.printMode)
        settings.setValue('PrintOrientation', self.printOrientation)
        settings.setValue('attachDistance', self.attachDistance)
        # position et état des barres d'outils :
        settings.setValue('toolsWindowGeometry', self.toolsWindow.saveGeometry())
        settings.setValue('toolsWindowState', self.toolsWindow.saveState())
        # ToolsKidWindow :
        settings.setValue('iconSizeKid', self.iconSizeKid)
        settings.setValue('toolsKidWindowGeometry', self.toolsKidWindowGeometry)
        settings.setValue('toolsKidWindowState', self.toolsKidWindowState)
        settings.setValue('toolsKidWindowWidth', self.toolsKidWindowWidth)
        settings.setValue('toolsKidWindowHeight', self.toolsKidWindowHeight)
        settings.setValue('toolsKidWindowX', self.toolsKidWindowX)
        settings.setValue('toolsKidWindowY', self.toolsKidWindowY)
        # visibilité des actions :
        for toolBar in self.toolsWindow.toolBarsList:
            for action in self.toolsWindow.toolBars[toolBar]:
                if action in self.toolsWindow.actionsState:
                    if isinstance(action, QtGui.QWidgetAction):
                        actionName = action.defaultWidget().toolTip()
                    elif isinstance(action, QtGui.QAction):
                        actionName = action.objectName()
                    if self.toolsWindow.actionsState[action][1]:
                        settings.setValue(utils.u('{0}/{1}').format(toolBar.objectName(), actionName), 1)
                    else:
                        settings.setValue(utils.u('{0}/{1}').format(toolBar.objectName(), actionName), 0)
                    if self.toolsWindow.actionsState[action][2]:
                        settings.setValue(utils.u('toolBarKid/{0}').format(actionName), 1)
                    else:
                        settings.setValue(utils.u('toolBarKid/{0}').format(actionName), 0)
        # blocage des unités (règle et équerre) :
        if self.toolsWindow.actionUnitsLock.isChecked():
            settings.setValue('unitsLocked', 1)
        else:
            settings.setValue('unitsLocked', 0)
        settings.setValue('instrumentsScale', self.instrumentsScale)
        # les pinceaux personnalisés :
        for i in range(5):
            a = 'pen{0}/'.format(i + 1)
            settings.setValue(a + 'Width', self.view.pens[i].width())
            settings.setValue(a + 'Color', self.view.pens[i].color())
            settings.setValue(a + 'Style', self.view.pens[i].style())
        # police :
        settings.setValue('font', self.view.font)
        settings.setValue('fontPointSize', self.view.font.pointSize())
        # les couleurs personnalisés :
        for action in self.toolsWindow.actionCustomColors:
            settings.setValue('custom/{0}'.format(action.objectName()), action.data())
        # les tailles personnalisés :
        for action in self.toolsWindow.actionCustomSizes:
            settings.setValue('custom/{0}'.format(action.objectName()), action.data())

    def configure(self):
        """
        Appel de la fenêtre de configuration du programme.
        """
        try:
            self.toolsWindow.hide()
            dialog = utils_config.ConfigurationDlg(parent=self)
            if dialog.exec_() == QtGui.QDialog.Accepted:
                self.toolsWindow.reloadToolBars()
                self.setWindowFlags(self.defaultFlags)
                self.calculDecalage()
                sg = QtGui.QApplication.desktop().screenGeometry(utils.SCREEN_NUMBER)
                size = sg.size()
                if utils.SCREEN_MODE == 'TESTS':
                    size = QtCore.QSize(500, 300)
                self.resize(size)
                pos = sg.topLeft()
                self.move(pos)
                self.setWindowFlags(self.defaultFlags | QtCore.Qt.FramelessWindowHint)
        finally:
            self.doRestore()
            self.toolsWindow.show()
            self.toolsWindow.repaint()

    def createDesktopFileLinux(self):
        try:
            result = False
            self.toolsWindow.hide()
            if sys.platform != 'linux2':
                print('NOT FOR THIS OS!')
                return
            title = QtCore.QCoreApplication.translate(
                'MainWindow', "Choose the Directory where the desktop file will be created")
            directory = QtGui.QFileDialog.getExistingDirectory(
                self,
                title,
                QtCore.QDir.homePath(),
                QtGui.QFileDialog.DontResolveSymlinks | QtGui.QFileDialog.ShowDirsOnly)
            if directory == '':
                return
            mustReplace = True
            # on remplacera CHEMIN par le chemin de l'installation (en enlevant le dernier /verac) :
            replaceWith = utils.u(self.beginDir)
            # on ouvre le fichier livré avec l'archive :
            desktopFileName = utils.u(self.beginDir + '/libs/Pylote.desktop')
            desktopFile = QtCore.QFile(desktopFileName)
            if not(desktopFile.open(QtCore.QFile.ReadOnly | QtCore.QFile.Text)):
                return
            stream = QtCore.QTextStream(desktopFile)
            stream.setCodec("UTF-8")
            lines = stream.readAll()
            # on remplace CHEMIN :
            lines = lines.replace('/CHEMIN', replaceWith)
            desktopFile.close()
            desktopFileName = utils.u('{0}/Pylote.desktop').format(directory)
            if mustReplace:
                QtCore.QFile(desktopFileName).remove()
            import io
            desktopFile = io.open(desktopFileName, 'wt', encoding='utf-8')
            for line in lines:
                desktopFile.write(line)
            desktopFile.close()
            QtCore.QFile(desktopFileName).setPermissions(\
                QtCore.QFile(desktopFileName).permissions() | \
                QtCore.QFile.ExeOwner | \
                QtCore.QFile.ExeUser | \
                QtCore.QFile.ExeGroup | \
                QtCore.QFile.ExeOther)
            result = True
        finally:
            if result == False:
                print('ERROR IN createDesktopFileLinux')
            self.toolsWindow.show()

    def createActions(self):
        """
        Ces actions sont là car elles sont disponibles depuis la trayicon
        """
        self.actionQuit = QtGui.QAction(
            QtCore.QCoreApplication.translate('MainWindow', 'Exit'), self)
        self.actionQuit.setShortcut(
            QtCore.QCoreApplication.translate('MainWindow', 'Ctrl+Q'))
        self.actionQuit.setIcon(QtGui.QIcon('images/application-exit.png'))
        self.connect(self.actionQuit, QtCore.SIGNAL('triggered()'), self.quit)

        self.actionHelp = QtGui.QAction(
            QtCore.QCoreApplication.translate('MainWindow', 'Help'), self)
        self.actionHelp.setShortcut(
            QtCore.QCoreApplication.translate('MainWindow', 'F1'))
        self.actionHelp.setIcon(QtGui.QIcon('images/help.png'))
        self.connect(self.actionHelp, QtCore.SIGNAL('triggered()'), self.helpPage)

        self.actionAbout = QtGui.QAction(
            QtCore.QCoreApplication.translate('MainWindow', 'About'), self)
        self.actionAbout.setIcon(QtGui.QIcon('images/help-about.png'))
        self.connect(self.actionAbout, QtCore.SIGNAL('triggered()'), self.about)

        self.actionNewScreenshot = QtGui.QAction(
            QtCore.QCoreApplication.translate('MainWindow', 'newScreenshot'), self)
        self.actionNewScreenshot.setIcon(QtGui.QIcon('images/camera.png'))
        QtCore.QObject.connect(self.actionNewScreenshot,
                QtCore.SIGNAL('triggered()'), self.newScreenshot)

        self.minimizeAction = QtGui.QAction(
            QtCore.QCoreApplication.translate('MainWindow', 'Minimize'), self)
        QtCore.QObject.connect(self.minimizeAction,
                QtCore.SIGNAL('triggered()'), self.doMinimize)

        self.restoreAction = QtGui.QAction(
            QtCore.QCoreApplication.translate('MainWindow', 'Restore'), self)
        QtCore.QObject.connect(self.restoreAction,
                QtCore.SIGNAL('triggered()'), self.doRestore)

    def createTrayIcon(self):
        """
        Création de la trayicon et de son menu (obtenu par clic droit).
        """
        self.trayIconMenu = QtGui.QMenu(self)
        self.trayIconMenu.addAction(self.actionHelp)
        self.trayIconMenu.addAction(self.actionAbout)
        self.trayIconMenu.addSeparator()
        self.trayIconMenu.addAction(self.actionNewScreenshot)
        self.trayIconMenu.addSeparator()
        self.trayIconMenu.addAction(self.minimizeAction)
        self.trayIconMenu.addAction(self.restoreAction)
        self.trayIconMenu.addSeparator()
        self.trayIconMenu.addAction(self.actionQuit)
        self.trayIcon = QtGui.QSystemTrayIcon(self)
        self.trayIcon.setContextMenu(self.trayIconMenu)

    def iconActivated(self, reason):
        """
        Séparation entre clic et double-clic (grâce à la variable iconDoubleClick).
        Un double-clic relance le screenshot.
        Le simple clic est géré après.
        """
        self.iconDoubleClick = False
        if reason == QtGui.QSystemTrayIcon.DoubleClick:
            self.iconDoubleClick = True
            self.newScreenshot()
        elif reason == QtGui.QSystemTrayIcon.Trigger:
            QtCore.QTimer.singleShot(500, self.iconClick)

    def iconClick(self):
        """
        Si on a bien à faire à un simple clic,
            on minimise ou restaure l'application.
        """
        if self.iconDoubleClick == False:
            if self.IsMinimized:
                self.doRestore()
            else:
                self.doMinimize()

    def doInstrumentLeftButtonMode(self, newMode):
        """
        On modifie l'effet du bouton gauche (mode TBI)
        """
        for instrument in self.listeInstruments:
            instrument.doLeftButtonMode(newMode)

    def units(self):
        what = self.sender().objectName()
        if what == 'UnitsSave':
            self.instrumentsScale = self.ruler.scale()
        elif what == 'RestoreUnits':
            for instrument in self.listeInstruments:
                if instrument.units:
                    instrument.setScale(self.instrumentsScale)
        elif what == 'UnitsInit':
            #self.instrumentsScale = 1
            for instrument in self.listeInstruments:
                if instrument.units:
                    instrument.setScale(1)

    def isDrawingEnabled(self):
        """
        Teste si on est autorisé à dessiner.
        Par exemple, si on déplace un instrument, la réponse est NON.
        """
        reponse = True
        for instrument in self.listeInstruments:
            if instrument.isUsed:
                reponse = False
        if self.toolsWindow.actionSelect.isChecked():
            reponse = False
        return reponse

    def eraseAll(self, doSave=True):
        """
        Efface tous les items.
        """
        for item in self.view.scene().items():
            if not(item in self.listeInstruments):
                try:
                    data = int(item.data(QtCore.Qt.UserRole))
                    self.view.scene().removeItem(item)
                    del item
                except:
                    pass
        if doSave:
            self.saveTempFile()

    def removeLast(self):
        """
        Efface le dernier item.
        """
        draws = []
        for item in self.view.scene().items():
            if not(item in self.listeInstruments):
                try:
                    data = int(item.data(QtCore.Qt.UserRole))
                    draws.append((item, data))
                except:
                    pass
        draws = sorted(draws, key=lambda data: data[1])
        if len(draws) > 0:
            item = draws[-1][0]
            self.view.scene().removeItem(item)
            del item
            self.saveTempFile()

    def removeSelected(self):
        """
        Supprime l'item sélectionné, après demande de confirmation.
        """
        items = self.view.scene().selectedItems()
        if len(items) > 0:
            item = items[0]
            if not(item in self.listeInstruments):
                try:
                    self.toolsWindow.hide()
                    data = int(item.data(QtCore.Qt.UserRole))
                    if QtGui.QMessageBox.question(
                        self, utils.PROGLABEL,
                        QtCore.QCoreApplication.translate('MainWindow', 'Delete this item ?'),
                        QtGui.QMessageBox.Yes|QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes:
                        self.view.scene().removeItem(item)
                        del item
                except:
                    pass
                finally:
                    self.saveTempFile()
                    self.toolsWindow.show()

    def editText(self):
        """
        Édition d'un texte ou du nom d'un point.
        """
        Reponse = False
        items = self.view.scene().selectedItems()
        if len(items) > 0:
            item = items[0]
            if isinstance(item, QtGui.QGraphicsTextItem):
                Reponse = True
                try:
                    self.toolsWindow.hide()
                    data = int(item.data(QtCore.Qt.UserRole))
                    initialText = item.toPlainText()
                    dialog = utils_dialogs.TextItemDlg(parent=self.view, graphicsTextItem=item)
                    if dialog.exec_() != QtGui.QDialog.Accepted:
                        item.setPlainText(initialText)
                except:
                    initialText = item.toPlainText()
                    text, ok = QtGui.QInputDialog.getText(
                        self, utils.PROGLABEL,
                        QtCore.QCoreApplication.translate('MainWindow', 'Point Name:'), 
                        QtGui.QLineEdit.Normal,
                        initialText)
                    if ok and text != '':
                        item.setPlainText(text)
                finally:
                    self.saveTempFile()
                    self.toolsWindow.show()
        return Reponse

    def whitePage(self):
        """
        Affiche une page blanche
        """
        self.backgroundPixmap = QtGui.QPixmap()
        self.brush.setTexture(self.backgroundPixmap)
        self.view.scene().setBackgroundBrush(self.brush)
        self.saveTempFile()

    def pointsPage(self):
        """
        Affiche une page de type papier pointé
        """
        image = QtGui.QImage('images/backgrounds/dots.svg')
        self.backgroundPixmap = QtGui.QPixmap.fromImage(image)
        self.brush.setTexture(self.backgroundPixmap)
        self.view.scene().setBackgroundBrush(self.brush)
        self.saveTempFile()

    def gridPage(self):
        """
        Affiche une grille
        """
        image = QtGui.QImage('images/backgrounds/grid.svg')
        self.backgroundPixmap = QtGui.QPixmap.fromImage(image)
        self.brush.setTexture(self.backgroundPixmap)
        self.view.scene().setBackgroundBrush(self.brush)
        self.saveTempFile()

    def chooseBackGround(self):
        """
        On sélectionne une image de fond
        """
        try:
            self.toolsWindow.hide()
            fileName = QtGui.QFileDialog.getOpenFileName(
                self, 
                QtCore.QCoreApplication.translate('MainWindow', 'Open Image'),
                self.backgroundsDir, 
                QtCore.QCoreApplication.translate('MainWindow', 'Image Files (*.png *.jpg *.jpeg *.svg)'))
            # pour PySide :
            if isinstance(fileName, tuple):
                fileName = fileName[0]
            if fileName == '':
                return
            self.backgroundsDir = QtCore.QFileInfo(fileName).absolutePath()
            image = QtGui.QImage(fileName)
            self.backgroundPixmap = QtGui.QPixmap.fromImage(image)
            brushTransform = QtGui.QTransform()
            brushTransform.translate(-self.decalX, -self.decalY)
            self.brush.setTransform(brushTransform)
            self.brush.setTexture(self.backgroundPixmap)
            self.view.scene().setBackgroundBrush(self.brush)
            self.saveTempFile()
        finally:
            self.toolsWindow.show()

    def fileSave(self):
        if self.fileName == '':
            self.fileSaveAs()
        else:
            try:
                self.toolsWindow.hide()
                self.saveFile(self.fileName)
            finally:
                self.toolsWindow.show()

    def selectFileName(self, title, proposedName, extension):
        fileName = QtGui.QFileDialog.getSaveFileName(self, title, proposedName, extension)
        # pour PySide :
        if isinstance(fileName, tuple):
            fileName = fileName[0]
        return fileName

    def fileSaveAs(self):
        """
        Enregistrement d'un fichier.
        On sauve le titre du programme en entête, 
            le fond et les items (sauf les instruments).
        """
        try:
            self.toolsWindow.hide()
            fileName = self.selectFileName(
                QtCore.QCoreApplication.translate('MainWindow', 'Save File'), 
                self.filesDir, 
                self.appExt)
            if fileName == '':
                return
            utils.doWaitCursor()
            fileName = utils.verifieExtension(fileName, 'plt')
            self.saveFile(fileName)
            self.filesDir = QtCore.QFileInfo(fileName).absolutePath()
            self.fileName = fileName
            baseName = QtCore.QFileInfo(fileName).baseName()
            title = utils.u('{0} [{1}]').format(
                QtCore.QCoreApplication.translate('MainWindow', 'Tools'), baseName)
            self.toolsWindow.setWindowTitle(title)
        finally:
            utils.restoreCursor()
            self.toolsWindow.show()

    def saveFile(self, fileName):
        outFile = QtCore.QFile(fileName)
        if not outFile.open(QtCore.QFile.WriteOnly):
            QtGui.QMessageBox.warning(
                self, utils.PROGLABEL,
                QtCore.QCoreApplication.translate(
                    'MainWindow', 'Failed to save\n  {0}').format(self.fileName))
            return
        try:
            stream = QtCore.QDataStream(outFile)
            stream.setVersion(QtCore.QDataStream.Qt_4_2)
            stream.writeQString('{0}_{1}'.format(utils.PROGNAME, utils.PROGVERSION))
            stream << self.brush
            # on trie pour enregistrer dans l'ordre de création :
            draws = []
            for item in self.view.scene().items():
                if not(item in self.listeInstruments):
                    try:
                        data = int(item.data(QtCore.Qt.UserRole))
                        draws.append((item, data))
                    except:
                        pass
            draws = sorted(draws, key=lambda data: data[1])
            for (item, data) in draws:
                self.writeItemToStream(stream, item)
        except:
            QtGui.QMessageBox.warning(
                self, utils.PROGLABEL,
                QtCore.QCoreApplication.translate(
                    'MainWindow', 'Failed to save\n  {0}').format(self.fileName))
        finally:
            outFile.close()

    def writeItemToStream(self, stream, item):
        """
        Pour chaque item, on enregistre son type (Point, ...) 
            puis les attributs utiles
        """
        if isinstance(item, utils_instruments.PointItem):
            stream.writeQString('Point')
            stream << item.pos() << item.transform() \
                   << item.pen() << item.brush() << item.font
            stream.writeQString(item.text)
            stream << item.textItem.pos()
        elif isinstance(item, QtGui.QGraphicsTextItem):
            stream.writeQString('Text')
            stream << item.pos() << item.transform() \
                    << item.font()  << item.defaultTextColor()
            stream.writeQString(item.toPlainText())
        elif isinstance(item, QtGui.QGraphicsPixmapItem):
            stream.writeQString('Pixmap')
            stream << item.pos() << item.transform() << item.pixmap()
        elif isinstance(item, QtGui.QGraphicsPathItem):
            stream.writeQString('Path')
            stream << item.pos() << item.transform() \
                   << item.pen() << item.path()
        elif isinstance(item, QtGui.QGraphicsLineItem):
            stream.writeQString('Line')
            stream << item.pos() << item.transform() \
                    << item.pen() << item.line()

    def saveTempFile(self):
        self.tempFileNum += 1
        if self.tempFileNum < 10:
            fileName = utils.u('{0}/000{1}.plt').format(self.tempPath, self.tempFileNum)
        elif self.tempFileNum < 100:
            fileName = utils.u('{0}/00{1}.plt').format(self.tempPath, self.tempFileNum)
        elif self.tempFileNum < 1000:
            fileName = utils.u('{0}/0{1}.plt').format(self.tempPath, self.tempFileNum)
        else:
            fileName = utils.u('{0}/{1}.plt').format(self.tempPath, self.tempFileNum)
        self.saveFile(fileName)

    def undo(self):
        self.tempFileNum -= 1
        if self.tempFileNum < 0:
            self.tempFileNum = 0
        if self.tempFileNum < 10:
            fileName = utils.u('{0}/000{1}.plt').format(self.tempPath, self.tempFileNum)
        elif self.tempFileNum < 100:
            fileName = utils.u('{0}/00{1}.plt').format(self.tempPath, self.tempFileNum)
        elif self.tempFileNum < 1000:
            fileName = utils.u('{0}/0{1}.plt').format(self.tempPath, self.tempFileNum)
        else:
            fileName = utils.u('{0}/{1}.plt').format(self.tempPath, self.tempFileNum)
        self.openFile(fileName, doSave=False)

    def redo(self):
        self.tempFileNum += 1
        if self.tempFileNum < 10:
            fileName = utils.u('{0}/000{1}.plt').format(self.tempPath, self.tempFileNum)
        elif self.tempFileNum < 100:
            fileName = utils.u('{0}/00{1}.plt').format(self.tempPath, self.tempFileNum)
        elif self.tempFileNum < 1000:
            fileName = utils.u('{0}/0{1}.plt').format(self.tempPath, self.tempFileNum)
        else:
            fileName = utils.u('{0}/{1}.plt').format(self.tempPath, self.tempFileNum)
        if QtCore.QFileInfo(fileName).exists():
            self.openFile(fileName, doSave=False)
        else:
            self.tempFileNum -= 1

    def fileOpen(self):
        """
        Ouverture d'un fichier.
        On sélectionne puis on délègue à la fonction fileReload.
        """
        try:
            self.toolsWindow.hide()
            fileName = QtGui.QFileDialog.getOpenFileName(
                self,
                QtCore.QCoreApplication.translate('MainWindow', 'Open File'),
                self.filesDir,
                self.appExt)
            # pour PySide :
            if isinstance(fileName, tuple):
                fileName = fileName[0]
            if fileName == '':
                return
            self.fileName = fileName
            self.fileReload()
        finally:
            self.toolsWindow.show()

    def fileReload(self):
        """
        Réouverture du fichier. On commence par effacer tout.
        On vérifie l'entête, puis on récupère le fond et les items.
        """
        if self.fileName != '':
            self.filesDir = QtCore.QFileInfo(self.fileName).absolutePath()
            self.openFile(self.fileName)
            baseName = QtCore.QFileInfo(self.fileName).baseName()
            title = utils.u('{0} [{1}]').format(
                QtCore.QCoreApplication.translate('MainWindow', 'Tools'), baseName)
            self.toolsWindow.setWindowTitle(title)
        else:
            self.eraseAll()
        if self.toolsWindow.actionSelect.isChecked():
            self.toolsWindow.select()

    def openFile(self, fileName, doSave=True):
        inFile = None
        try:
            inFile = QtCore.QFile(fileName)
            inFile.open(QtCore.QIODevice.ReadOnly)
            self.eraseAll(doSave=doSave)
            stream = QtCore.QDataStream(inFile)
            stream.setVersion(QtCore.QDataStream.Qt_4_2)
            progNameVersion = ''
            progNameVersion = stream.readQString()
            if not(utils.PROGNAME in progNameVersion):
                QtGui.QMessageBox.warning(
                    self, utils.PROGLABEL,
                    QtCore.QCoreApplication.translate('MainWindow', 'not a valid file'))
                return
            oldFile = False
            if progNameVersion == utils.PROGNAME:
                oldFile = True
            self.readBackgroundFromStream(stream, oldFile)
            while not inFile.atEnd():
                self.readItemFromStream(stream, oldFile)
        except:
            QtGui.QMessageBox.warning(
                self, utils.PROGLABEL,
                QtCore.QCoreApplication.translate('MainWindow', 'Failed to open\n  {0}').format(fileName))
        finally:
            if inFile != None:
                inFile.close()

    def fileGoPrevious(self):
        if self.fileName != '':
            listFiles = []
            dirIterator = QtCore.QDirIterator(self.filesDir, 
                                            ['*.plt'], 
                                            QtCore.QDir.Files)
            while dirIterator.hasNext():
                listFiles.append(dirIterator.next())
            listFiles = sorted(listFiles)
            i = listFiles.index(self.fileName)
            if i > 0:
                self.fileName = listFiles[i - 1]
                self.fileReload()

    def fileGoNext(self):
        if self.fileName != '':
            listFiles = []
            dirIterator = QtCore.QDirIterator(self.filesDir, 
                                            ['*.plt'], 
                                            QtCore.QDir.Files)
            while dirIterator.hasNext():
                listFiles.append(dirIterator.next())
            listFiles = sorted(listFiles)
            i = listFiles.index(self.fileName)
            if i < len(listFiles) - 1:
                self.fileName = listFiles[i + 1]
                self.fileReload()

    def exportToPainter(self, what, export='SVG'):
        """
        Crée un painter pour le paramètre what et dessine dessus.
        Retourne le painter.
        what peut être un générateur de SVG, un printer (imprimante ou PDF)
        """
        if export == 'SVG':
            scale = 1
        else:
            if self.printMode == 'FullPage':
                scale = what.pageRect().width() / self.view.width()
                yscale = what.pageRect().height() / self.view.height()
                if yscale < scale:
                    scale = yscale
            else:
                paperWidth = what.paperSize(QtGui.QPrinter.Millimeter).width()
                pageWidth = what.pageRect(QtGui.QPrinter.Millimeter).width()\
                            - what.pageRect(QtGui.QPrinter.Millimeter).left()
                scale = paperWidth / pageWidth
                #print('paperWidth :', paperWidth)
                #print('pageWidth :', pageWidth)
                #print('scale :', scale)

        def drawTextToPainter(painter, textItem):
            """
            la taille et la position sont modifiées (pourquoi et comment ?)
            Les 2 variables ont été trouvés par tests (mais sont elles universelles ?)
            """
            if export == 'SVG':
                decalageFont = 3
                coeffFont = 1.3333
            elif export in ('PDF', 'PRINT'):
                decalageFont = 5
                coeffFont = 1.02
            painter.setPen(QtGui.QPen(textItem.defaultTextColor()))
            font = textItem.font()
            font.setPointSizeF(font.pointSize() * coeffFont / scale)
            painter.setFont(font)
            painter.translate(textItem.pos())
            boundingRect = textItem.boundingRect()
            boundingRect.setWidth(boundingRect.width() * scale)
            boundingRect.setHeight(boundingRect.height() * scale)
            boundingRect.setX(decalageFont)
            boundingRect.setY(decalageFont)
            painter.drawText(boundingRect, textItem.toPlainText())

        painter = QtGui.QPainter(what)
        painter.scale(scale, scale)
        # vérifier encore le décalage :
        origineX = self.view.width() // 2 - self.decalX
        origineY = self.view.height() // 2 - self.decalY
        brushWidth = self.brush.texture().width()
        brushHeight = self.brush.texture().height()
        columns = self.view.width() // brushWidth
        rows = self.view.height() // brushHeight
        for i in range(columns):
            for j in range(rows):
                decalage = QtCore.QPoint(origineX + i * brushWidth, origineY + j * brushHeight)
                painter.drawPixmap(decalage, self.brush.texture())

        draws = []
        for item in self.view.scene().items():
            if not(item in self.listeInstruments):
                try:
                    data = int(item.data(QtCore.Qt.UserRole))
                    draws.append((item, data))
                except:
                    pass
        draws = sorted(draws, key=lambda data: data[1])
        for (item, data) in draws:
            painter.resetTransform()
            painter.scale(scale, scale)
            painter.translate(QtCore.QPoint(self.size().width() // 2, 
                                            self.size().height() // 2))
            if isinstance(item, utils_instruments.PointItem):
                painter.translate(item.pos())
                painter.setPen(item.pen())
                painter.setBrush(item.brush())
                painter.drawPath(item.path)
                drawTextToPainter(painter, item.textItem)
            elif isinstance(item, QtGui.QGraphicsTextItem):
                drawTextToPainter(painter, item)
            elif isinstance(item, QtGui.QGraphicsPixmapItem):
                painter.drawPixmap(item.pos(), item.pixmap())
            elif isinstance(item, QtGui.QGraphicsPathItem):
                painter.translate(item.pos())
                painter.setPen(item.pen())
                painter.setBrush(item.brush())
                painter.drawPath(item.path())
            elif isinstance(item, QtGui.QGraphicsLineItem):
                painter.translate(item.pos())
                painter.setPen(item.pen())
                painter.drawLine(item.line())
        painter.end()

    def createPrinter(self, fileName='', direct=False):
        """
        Initialisation d'un printer.
        Si fileName != '', on fait un PDF
        """
        result = False
        utils.doWaitCursor()
        try:
            printer = QtGui.QPrinter(QtGui.QPrinter.ScreenResolution)#QtGui.QPrinter.HighResolution)
            printer.setPaperSize(QtGui.QPrinter.A4)
            #printer.setOutputFormat(QtGui.QPrinter.PdfFormat)
            printer.setOutputFormat(QtGui.QPrinter.NativeFormat)
            if self.printOrientation == 'Portrait':
                printer.setOrientation(QtGui.QPrinter.Portrait)
            else:
                printer.setOrientation(QtGui.QPrinter.Landscape)
            #printer.setFullPage(True)
            #printer.setPageMargins(10.0, 10.0, 10.0, 10.0, QtGui.QPrinter.Millimeter)
            result = printer
            if fileName != '':
                # on veut un PDF :
                printer.setOutputFormat(QtGui.QPrinter.PdfFormat)
                printer.setOutputFileName(fileName)
            elif not(direct):
                # on lance le dialog d'impression :
                utils.restoreCursor()
                printDialog = QtGui.QPrintDialog(printer)
                if printDialog.exec_() != QtGui.QDialog.Accepted:
                    result = False
                utils.doWaitCursor()
        finally:
            utils.restoreCursor()
            return result

    def exportSvg(self):
        """
        Export du fichier en svg, pour réutilisation dans Inkscape par exemple.
        """
        try:
            self.toolsWindow.hide()
            self.svgTitle = QtCore.QCoreApplication.translate(
                'MainWindow', 'Export as SVG File')
            self.svgExt = QtCore.QCoreApplication.translate(
                'MainWindow', 'SVG Files (*.svg)')
            if self.fileName != '':
                baseName = QtCore.QFileInfo(self.fileName).baseName()
                proposedName = utils.u('{0}/{1}.svg').format(self.filesDir, baseName)
            else:
                proposedName = self.filesDir
            fileName = self.selectFileName(self.svgTitle, proposedName, self.svgExt)
            if fileName == '':
                return
            fileName = utils.verifieExtension(fileName, 'svg')
            self.filesDir = QtCore.QFileInfo(fileName).absolutePath()
            generator = QtSvg.QSvgGenerator()
            generator.setFileName(fileName)
            generator.setSize(self.size())
            self.exportToPainter(generator)
        finally:
            self.toolsWindow.show()

    def exportPdf(self):
        try:
            self.toolsWindow.hide()
            self.pdfTitle = QtCore.QCoreApplication.translate(
                'MainWindow', 'Export as PDF File')
            self.pdfExt = QtCore.QCoreApplication.translate(
                'MainWindow', 'PDF Files (*.pdf)')
            if self.fileName != '':
                baseName = QtCore.QFileInfo(self.fileName).baseName()
                proposedName = utils.u('{0}/{1}.pdf').format(self.filesDir, baseName)
            else:
                proposedName = self.filesDir
            fileName = self.selectFileName(self.pdfTitle, proposedName, self.pdfExt)
            if fileName == '':
                return
            fileName = utils.verifieExtension(fileName, 'pdf')
            self.filesDir = QtCore.QFileInfo(fileName).absolutePath()
            printer = self.createPrinter(fileName)
            if printer == False:
                return
            self.exportToPainter(printer, export='PDF')
        finally:
            self.toolsWindow.show()

    def documentPrint(self):
        try:
            self.toolsWindow.hide()
            printer = self.createPrinter()
            if printer == False:
                return
            self.exportToPainter(printer, export='PRINT')
        finally:
            self.toolsWindow.show()

    def documentPrintDirect(self):
        printer = self.createPrinter(direct=True)
        if printer == False:
            return
        self.exportToPainter(printer, export='PRINT')













    def readBackgroundFromStream(self, stream, oldFile=False):
        """
        On applique le fond du fichier
        """
        stream >> self.brush
        brushTransform = QtGui.QTransform()
        brushTransform.translate(-self.decalX, -self.decalY)
        self.brush.setTransform(brushTransform)
        self.view.scene().setBackgroundBrush(self.brush)

    def readItemFromStream(self, stream, oldFile=False):
        """
        Récupération des items et création
        """
        itemType = ''
        position = QtCore.QPointF()
        transform = QtGui.QTransform()
        itemType = stream.readQString()
        if oldFile:
            matrix = QtGui.QMatrix()
            stream >> position >> matrix
        else:
            stream >> position >> transform
        if itemType == 'Point':
            pen = QtGui.QPen()
            brush = QtGui.QBrush()
            font = QtGui.QFont()
            text = ''
            textPos = QtCore.QPointF()
            stream >> pen >> brush >> font
            text = stream.readQString()
            stream >> textPos
            pointItem = utils_instruments.PointItem(self, pen, brush, font, text)
            if text != '':
                pointItem.textItem.setPos(textPos)
            self.view.scene().addItem(pointItem)
            pointItem.setPos(position)
            self.view.drawId += 1
            pointItem.setData(QtCore.Qt.UserRole, self.view.drawId)
        elif itemType == 'Text':
            font = QtGui.QFont()
            color = QtGui.QColor()
            text = ''
            stream >> font >> color
            text = stream.readQString()
            textItem = QtGui.QGraphicsTextItem()
            textItem.setPlainText(text)
            textItem.setFont(font)
            self.view.scene().addItem(textItem)
            textItem.setDefaultTextColor(color)
            textItem.setPos(position)
            self.view.drawId += 1
            textItem.setData(QtCore.Qt.UserRole, self.view.drawId)
        elif itemType == 'Pixmap':
            pixmap = QtGui.QPixmap()
            stream >> pixmap
            pixmapItem = QtGui.QGraphicsPixmapItem(pixmap)
            self.view.scene().addItem(pixmapItem)
            pixmapItem.setPos(position)
            self.view.drawId += 1
            pixmapItem.setData(QtCore.Qt.UserRole, self.view.drawId)
        elif itemType == 'Path':
            pen = QtGui.QPen()
            brush = QtGui.QBrush(QtCore.Qt.green, QtCore.Qt.NoBrush)
            path = QtGui.QPainterPath()
            stream >> pen >> path
            last = self.view.scene().addPath(path, pen, brush)
            last.setPos(position)
            self.view.drawId += 1
            last.setData(QtCore.Qt.UserRole, self.view.drawId)
        elif itemType == 'Line':
            pen = QtGui.QPen()
            line = QtCore.QLineF()
            stream >> pen >> line
            last = self.view.scene().addLine(line, pen)
            last.setPos(position)
            self.view.drawId += 1
            last.setData(QtCore.Qt.UserRole, self.view.drawId)

    def about(self):
        """
        Affiche la fenêtre À propos
        """
        try:
            self.toolsWindow.hide()
            import utils_about
            aboutdialog = utils_about.AboutDlg(self, self.lang, icon='./images/icon.png')
            aboutdialog.exec_()
        finally:
            self.toolsWindow.show()

    def helpPage(self):
        """
        Affiche la page d'aide dans le navigateur.
        """
        theUrl = utils.HELPPAGE
        utils.webhelp_in_browser(theUrl)






"""
###########################################################
###########################################################

                LANCEMENT DU PROGRAMME

###########################################################
###########################################################
"""

if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)

    ###########################################
    # Installation de l'internationalisation :
    ###########################################
    locale = QtCore.QLocale.system().name()
    LANG = locale.split('_')[0]
    # recherche d'un i18n passé en argument (par exemple LANG=fr_FR) :
    for arg in sys.argv:
        if arg.split('=')[0] == 'LANG':
            locale = arg.split('=')[1]
            LANG = locale.split('_')[0]
    print('locale:', locale, 'lang:', LANG)
    QtTranslationsPath = QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath)
    qtTranslator = QtCore.QTranslator()
    if qtTranslator.load('qt_' + locale, QtTranslationsPath):
        app.installTranslator(qtTranslator)
    trans_dir = QtCore.QDir('./translations')
    localefile = trans_dir.filePath('pylote_' + locale)
    appTranslator = QtCore.QTranslator()
    if appTranslator.load(localefile, ''):
        app.installTranslator(appTranslator)    

    app.setWindowIcon(QtGui.QIcon('./images/icon.png'))

    ##################################################
    # Définition de SCREEN_MODE et de WITH_TRAY_ICON :
    ##################################################
    # on peut passer SCREEN_MODE en argument (utile pour TESTS)
    for arg in sys.argv:
        if arg in ('FULL_SCREEN', 'FULL_SPACE', 'TESTS'):
            utils.changeScreenMode(sys.argv[1])
    # valeur par défaut suivant l'os :
    if utils.SCREEN_MODE != 'TESTS':
        settings = QtCore.QSettings(utils.PROGNAME, 'config')
        if sys.platform == 'win32':
            utils.changeScreenMode(settings.value('SCREEN_MODE', 'FULL_SCREEN'))
        else:
            utils.changeScreenMode(settings.value('SCREEN_MODE', 'FULL_SPACE'))
    utils.myPrint('SCREEN_MODE:', utils.SCREEN_MODE)
    # pas de TrayIcon si le système ne le supporte pas :
    if not QtGui.QSystemTrayIcon.isSystemTrayAvailable():
        utils.changeWithTrayIcon(False)
    # pas besoin de TrayIcon en mode FULL_SCREEN :
    if utils.SCREEN_MODE == 'FULL_SCREEN':
        utils.changeWithTrayIcon(False)

    ##################################################
    # Création et lancement du programme :
    ##################################################
    mainWindow = MainWindow(appTranslator)
    # en fonction de SCREEN_MODE :
    if utils.SCREEN_MODE == 'TESTS':
        mainWindow.show()
    elif utils.SCREEN_MODE == 'FULL_SPACE':
        rect = QtGui.QApplication.desktop().availableGeometry(mainWindow.screenNumber)
        mainWindow.resize(int(rect.width()), int(rect.height()))
        mainWindow.showMaximized()
    elif utils.SCREEN_MODE == 'FULL_SCREEN':
        mainWindow.showFullScreen()
    else:
        mainWindow.showFullScreen()

    app.exec_()
    #sys.exit(app.exec_())

